@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # Copyright (c) 2009, Willow Garage, Inc.
 # All rights reserved.
 #
@@ -71,18 +70,25 @@ def read_issue(filename="/etc/issue"):
     return None
 
 
-def read_os_release(filename="/etc/os-release"):
+def read_os_release(filename=None):
     """
-    :returns: Dictonary of key value pairs from /etc/os-release, with quotes stripped from values
+    :returns: Dictionary of key value pairs from /etc/os-release or fallback to 
+      /usr/lib/os-release, with quotes stripped from values
     """
+    if filename is None:
+        filename = '/etc/os-release'
+        if not os.path.exists(filename):
+            filename = '/usr/lib/os-release'
+
+    if not os.path.exists(filename):
+        return None
+
     release_info = {}
-    if os.path.exists(filename):
-        with codecs.open(filename, 'r', encoding=locale.getpreferredencoding()) as f:
-            for line in f:
-                key, val = line.rstrip('\n').partition('=')[::2]
-                release_info[key] = val.strip('"')
-            return release_info
-    return None
+    with codecs.open(filename, 'r', encoding=locale.getpreferredencoding()) as f:
+        for line in f:
+            key, val = line.rstrip('\n').partition('=')[::2]
+            release_info[key] = val.strip('"')
+    return release_info
 
 
 class OsNotDetected(Exception):
@@ -106,14 +112,14 @@ class OsDetector(object):
 
     def get_version(self):
         """
-        :returns: standardized version for this OS. (ala Ubuntu Hardy Heron = "8.04")
+        :returns: standardized version for this OS. (aka Ubuntu Hardy Heron = "8.04")
         :raises: :exc:`OsNotDetected` if called on incorrect OS.
         """
         raise NotImplementedError("get_version unimplemented")
 
     def get_codename(self):
         """
-        :returns: codename for this OS. (ala Ubuntu Hardy Heron = "hardy").  If codenames are not available for this OS, return empty string.
+        :returns: codename for this OS. (aka Ubuntu Hardy Heron = "hardy").  If codenames are not available for this OS, return empty string.
         :raises: :exc:`OsNotDetected` if called on incorrect OS.
         """
         raise NotImplementedError("get_codename unimplemented")
@@ -160,6 +166,8 @@ class Debian(LsbDetect):
                 return 'jessie'
             if v.startswith('9.'):
                 return 'stretch'
+            if v.startswith('10.'):
+                return 'buster'
             return ''
 
 
@@ -206,6 +214,24 @@ class FdoDetect(OsDetector):
         raise OsNotDetected("called in incorrect OS")
 
 
+class OpenEmbedded(OsDetector):
+    """
+    Detect OpenEmbedded.
+    """
+    def is_os(self):
+        return "ROS_OS_OVERRIDE" in os.environ and os.environ["ROS_OS_OVERRIDE"] == "openembedded"
+
+    def get_version(self):
+        if self.is_os():
+            return ""
+        raise OsNotDetected('called in incorrect OS')
+
+    def get_codename(self):
+        if self.is_os():
+            return ""
+        raise OsNotDetected('called in incorrect OS')
+
+
 class OpenSuse(OsDetector):
     """
     Detect OpenSuse OS.
@@ -302,7 +328,7 @@ class Rhel(Fedora):
         raise OsNotDetected('called in incorrect OS')
 
 
-# Source: http://en.wikipedia.org/wiki/Mac_OS_X#Versions
+# Source: https://en.wikipedia.org/wiki/MacOS#Versions
 _osx_codename_map = {
     4: 'tiger',
     5: 'leopard',
@@ -313,7 +339,10 @@ _osx_codename_map = {
     10: 'yosemite',
     11: 'el capitan',
     12: 'sierra',
-    13: 'high sierra'}
+    13: 'high sierra',
+    14: 'mojave',
+    15: 'catalina',
+}
 
 
 def _osx_codename(major, minor):
@@ -667,6 +696,7 @@ class OsDetect:
         return self._os_codename
 
 
+OS_ALPINE = 'alpine'
 OS_ARCH = 'arch'
 OS_MANJARO = 'manjaro'
 OS_CENTOS = 'centos'
@@ -680,16 +710,22 @@ OS_FUNTOO = 'funtoo'
 OS_GENTOO = 'gentoo'
 OS_LINARO = 'linaro'
 OS_MINT = 'mint'
+OS_MX = 'mx'
 OS_NEON = 'neon'
+OS_OPENEMBEDDED = 'openembedded'
 OS_OPENSUSE = 'opensuse'
 OS_OPENSUSE13 = 'opensuse'
+OS_TIZEN = 'tizen'
 OS_OSX = 'osx'
 OS_QNX = 'qnx'
 OS_RHEL = 'rhel'
 OS_SLACKWARE = 'slackware'
 OS_UBUNTU = 'ubuntu'
+OS_CLEARLINUX = 'clearlinux'
+OS_NIXOS = 'nixos'
 OS_WINDOWS = 'windows'
 
+OsDetect.register_default(OS_ALPINE, FdoDetect("alpine"))
 OsDetect.register_default(OS_ARCH, Arch())
 OsDetect.register_default(OS_MANJARO, Manjaro())
 OsDetect.register_default(OS_CENTOS, Centos())
@@ -703,15 +739,20 @@ OsDetect.register_default(OS_FUNTOO, Funtoo())
 OsDetect.register_default(OS_GENTOO, Gentoo())
 OsDetect.register_default(OS_LINARO, LsbDetect("Linaro"))
 OsDetect.register_default(OS_MINT, LsbDetect("LinuxMint"))
+OsDetect.register_default(OS_MX, LsbDetect("MX"))
 OsDetect.register_default(OS_NEON, LsbDetect("neon"))
+OsDetect.register_default(OS_OPENEMBEDDED, OpenEmbedded())
 OsDetect.register_default(OS_OPENSUSE, OpenSuse())
 OsDetect.register_default(OS_OPENSUSE13, OpenSuse(brand_file='/etc/SUSE-brand'))
 OsDetect.register_default(OS_OPENSUSE, FdoDetect("opensuse"))
+OsDetect.register_default(OS_TIZEN, FdoDetect("tizen"))
 OsDetect.register_default(OS_OSX, OSX())
 OsDetect.register_default(OS_QNX, QNX())
 OsDetect.register_default(OS_RHEL, Rhel())
 OsDetect.register_default(OS_SLACKWARE, Slackware())
 OsDetect.register_default(OS_UBUNTU, LsbDetect("Ubuntu"))
+OsDetect.register_default(OS_CLEARLINUX, FdoDetect("clear-linux-os"))
+OsDetect.register_default(OS_NIXOS, FdoDetect("nixos"))
 OsDetect.register_default(OS_WINDOWS, Windows())
 
 
